# ================================================================

use strict;

# ================================================================

our($iLibraryVersion)='20211225';

# ================================================================

sub decodeAsURL
{
	my($aTarget)=@_;

	my($tResult)=$aTarget;

	$tResult=~ s/%([0-9A-Fa-f][0-9A-Fa-f])/pack('C',hex($1))/eg;
	$tResult=~ s/\+/ /g;

	return $tResult;
}

# ----------------------------------------------------------------

sub getDayOfWeek
{
	my($aYear,$aMonth,$aDay)=@_;

	if(($aMonth==1)||($aMonth)==2)
	{
		$aMonth+=12;
		$aYear--;
	}

	return ($aDay+$aYear+int($aYear/4)-int($aYear/100)+int($aYear/400)+int(($aMonth*13+8)/5))%7;
}

# ----------------------------------------------------------------

sub getDaysOfMonth
{
	my($aYear,$aMonth)=@_;

	my(@tDaysOfMonth)=(0,31,28,31,30,31,30,31,31,30,31,30,31);

	if(&isLeapYear($aYear))
	{
		$tDaysOfMonth[2]=29;
	}

	return $tDaysOfMonth[$aMonth];
}

# ----------------------------------------------------------------

sub getDomainURL
{
	my($tResult)='';

	if(&isSame($ENV{'HTTPS'},'on'))
	{
		$tResult='https://';
	}
	else
	{
		$tResult='http://';
	}

	$tResult.=$ENV{'HTTP_HOST'};

	if(&isDifferent(($ENV{'SERVER_PORT'},'80'))&&(&isDifferent($ENV{'SERVER_PORT'},'443')))
	{
		$tResult.=':'.$ENV{'SERVER_PORT'};
	}

	return $tResult;
}

# ----------------------------------------------------------------

sub getDumpIfExists
{
	my($aTarget)=@_;

	my($tResult)='';

	if(-e $aTarget)
	{
		my(@tDumpData)=&readFileAsText($aTarget);

		for(my($tCount1)=0;$tCount1<=($#tDumpData);$tCount1++)
		{
			$tResult.=$tDumpData[$tCount1];
		}
	}

	return $tResult;
}

# ----------------------------------------------------------------

sub getHTMLCaption
{
	my($aCaption,$aSize)=@_;

	my($tResult)='';

	$tResult.='<h'.$aSize.'>'.$aCaption.'</h'.$aSize.'>'."\n";

	return $tResult;
}

# ----------------------------------------------------------------

sub getHTMLDocumentCloseWithApplicationVersion
{
	my($aApplicationName,$aApplicationVersion,$aLibraryVersion)=@_;

	my($tResult)='';

	$tResult.=&getHTMLLine();
	$tResult.='<address>'.$aApplicationName.' version '.$aApplicationVersion.' (Library:'.$aLibraryVersion.') / Programmed by Kei Kishiro</address>'."\n";
	$tResult.='</div>'."\n";
	$tResult.='</body>'."\n";
	$tResult.='</html>'."\n";

	return $tResult;
}

# ----------------------------------------------------------------

sub getHTMLDocumentOpen
{
	my($aTitle,$aContentLanguage,$aCharset,$aCSSFiles,$aJavaScripts)=@_;

	my($tResult)='';

	$tResult.='<!doctype html public "-//W3C//DTD HTML 4.01 Transitional//EN">'."\n";
	$tResult.='<html lang="'.$aContentLanguage.'">'."\n";
	$tResult.='<head>'."\n";
	$tResult.='<meta name="viewport" content="width=device-width,initial-scale=1">'."\n";
	$tResult.='<meta name="ROBOTS" content="NOARCHIVE">'."\n";
	$tResult.='<meta http-equiv="Content-Type" content="text/html; charset='.$aCharset.'">'."\n";

	if(($#$aCSSFiles)>=0)
	{
		$tResult.='<style type="text/css">'."\n";

		for(my($tCount1)=0;$tCount1<=($#$aCSSFiles);$tCount1++)
		{
			$tResult.=&getDumpIfExists($$aCSSFiles[$tCount1]);
		}

		$tResult.='</style>'."\n";
	}

	if(($#$aJavaScripts)>=0)
	{
		$tResult.='<script type="text/javascript">'."\n";
		$tResult.='<!--'."\n";

		for(my($tCount1)=0;$tCount1<=($#$aJavaScripts);$tCount1++)
		{
			$tResult.=&getDumpIfExists($$aJavaScripts[$tCount1]);
		}

		$tResult.='//-->'."\n";
		$tResult.='</script>'."\n";
	}

	$tResult.='<title>'.$aTitle.'</title>'."\n";
	$tResult.='</head>'."\n";
	$tResult.='<body>'."\n";
	$tResult.='<div id="Outline">'."\n";

	return $tResult;
}

# ----------------------------------------------------------------

sub getHTMLLine
{
	my($tResult)='';

	$tResult.='<hr>'."\n";

	return $tResult;
}

# ----------------------------------------------------------------

sub getHTMLParagraphClose
{
	my($tResult)='';

	$tResult.='</p>'."\n";

	return $tResult;
}

# ----------------------------------------------------------------

sub getHTMLParagraphOpen
{
	my($tResult)='';

	$tResult.='<p>'."\n";

	return $tResult;
}

# ----------------------------------------------------------------

sub getHTTPContentHeader
{
	my($aContentLanguage,$aCharset)=@_;

	my($tResult)='';

	$tResult.='Content-type: text/html; charset='.$aCharset."\n";
	$tResult.='Content-Language: '.$aContentLanguage."\n";
	$tResult.='Cache-Control: no-cache'."\n";

	return $tResult;
}

# ----------------------------------------------------------------

sub getQueryEntries
{
	my($aString,$aDoDecode)=@_;

	my(@tQuery)=split(/\&/,$aString);
	my(%tReturn)=();

	for(my($tCount1)=0;$tCount1<=($#tQuery);$tCount1++)
	{
		my($tName,$tValue)=split(/=/,$tQuery[$tCount1]);
		chomp($tValue);

		if($aDoDecode)
		{
			$tReturn{$tName}=&decodeAsURL($tValue);
		}
		else
		{
			$tReturn{$tName}=$tValue;
		}
	}

	return %tReturn;
}

# ----------------------------------------------------------------

sub getScriptURL
{
	return &getDomainURL().$ENV{'SCRIPT_NAME'};
}

# ----------------------------------------------------------------

sub getURLQueryEntries
{
	my($aDoDecode)=@_;

	return &getQueryEntries($ENV{'QUERY_STRING'},$aDoDecode);
}

# ----------------------------------------------------------------

sub isDifferent
{
	my($aString1,$aString2)=@_;

	return ($aString1 ne $aString2);
}

# ----------------------------------------------------------------

sub isLeapYear
{
	my($aYear)=@_;

	if(($aYear%4)==0)
	{
		if(($aYear%100)==0)
		{
			if(($aYear%400)==0)
			{
				return 1;
			}
		}
		else
		{
			return 1;
		}
	}

	return 0;
}

# ----------------------------------------------------------------

sub isSame
{
	my($aString1,$aString2)=@_;

	return ($aString1 eq $aString2);
}

# ----------------------------------------------------------------

sub printResponseCache
{
	my($aHeader,$aBody)=@_;

	if((&isDifferent($aHeader,''))&&(&isDifferent($aBody,'')))
	{
		print $aHeader;
		print 'Content-Length: '.length($aBody)."\n\n";
		print $aBody;
	}
}

# ----------------------------------------------------------------

sub readFileAsText
{
	my($aTarget)=@_;

	open(DATA_FILE,'<'.$aTarget);
	flock(DATA_FILE,1);
	my(@tFileData)=<DATA_FILE>;
	close(DATA_FILE);

	return @tFileData;
}

# ================================================================

1;
